@@ -24,7 +24,7 @@ PUSHES_BLOCK = 2048  # starts a block (while, try, finally, with, etc.)
 POPS_BLOCK = 4096  # ends a block
 
 
-class Opcode(object):
+class Opcode:
   """An opcode without arguments."""
 
   __slots__ = ("line", "index", "prev", "next", "target", "block_target",
@@ -138,7 +138,7 @@ class OpcodeWithArg(Opcode):
   __slots__ = ("arg", "pretty_arg")
 
   def __init__(self, index, line, arg, pretty_arg):
-    super(OpcodeWithArg, self).__init__(index, line)
+    super().__init__(index, line)
     self.arg = arg
     self.pretty_arg = pretty_arg
 
@@ -319,6 +319,15 @@ class BEFORE_ASYNC_WITH(Opcode):
   __slots__ = ()
 
 
+class BEGIN_FINALLY(Opcode):
+  __slots__ = ()
+
+
+class END_ASYNC_FOR(Opcode):
+  FLAGS = HAS_JUNKNOWN | NO_NEXT  # might re-raise an exception
+  __slots__ = ()
+
+
 class STORE_MAP(Opcode):
   __slots__ = ()
 
@@ -831,7 +840,18 @@ class LOAD_METHOD(OpcodeWithArg):  # Arg: Index in name list
 
 
 class CALL_METHOD(OpcodeWithArg):  # Arg: #args
-  FLAGS = HAS_ARGUMENT
+  FLAGS = HAS_NARGS|HAS_ARGUMENT|HAS_JUNKNOWN
+  __slots__ = ()
+
+
+class CALL_FINALLY(OpcodeWithArg):  # Arg: Jump offset to finally block
+  FLAGS = HAS_JREL | HAS_ARGUMENT
+  __slots__ = ()
+
+
+class POP_FINALLY(OpcodeWithArg):
+  # might re-raise an exception or jump to a finally
+  FLAGS = HAS_ARGUMENT | HAS_JUNKNOWN
   __slots__ = ()
 
 
@@ -957,7 +977,7 @@ python2_mapping = {
     147: MAP_ADD,
 }
 
-python3_mapping = {
+python_3_5_mapping = {
     1: POP_TOP,
     2: ROT_TWO,
     3: ROT_THREE,
@@ -968,6 +988,8 @@ python3_mapping = {
     11: UNARY_NEGATIVE,
     12: UNARY_NOT,
     15: UNARY_INVERT,
+    16: BINARY_MATRIX_MULTIPLY,
+    17: INPLACE_MATRIX_MULTIPLY,
     19: BINARY_POWER,
     20: BINARY_MULTIPLY,
     22: BINARY_MODULO,
@@ -981,7 +1003,6 @@ python3_mapping = {
     50: GET_AITER,
     51: GET_ANEXT,
     52: BEFORE_ASYNC_WITH,
-    54: STORE_MAP,  # removed in Python 3.5.2
     55: INPLACE_ADD,
     56: INPLACE_SUBTRACT,
     57: INPLACE_MULTIPLY,
@@ -995,7 +1016,7 @@ python3_mapping = {
     66: BINARY_OR,
     67: INPLACE_POWER,
     68: GET_ITER,
-    69: STORE_LOCALS,  # removed in Python 3.4
+    69: GET_YIELD_FROM_ITER,
     70: PRINT_EXPR,
     71: LOAD_BUILD_CLASS,  # PRINT_ITEM in Python 2
     72: YIELD_FROM,  # PRINT_NEWLINE in Python 2
@@ -1006,7 +1027,8 @@ python3_mapping = {
     78: INPLACE_XOR,
     79: INPLACE_OR,
     80: BREAK_LOOP,
-    81: WITH_CLEANUP,
+    81: WITH_CLEANUP_START,
+    82: WITH_CLEANUP_FINISH,
     83: RETURN_VALUE,
     84: IMPORT_STAR,
     86: YIELD_VALUE,
@@ -1064,6 +1086,11 @@ python3_mapping = {
     146: SET_ADD,
     147: MAP_ADD,
     148: LOAD_CLASSDEREF,  # not in Python 2
+    149: BUILD_LIST_UNPACK,
+    150: BUILD_MAP_UNPACK,
+    151: BUILD_MAP_UNPACK_WITH_CALL,
+    152: BUILD_TUPLE_UNPACK,
+    153: BUILD_SET_UNPACK,
     154: SETUP_ASYNC_WITH,
 }
 
@@ -1071,21 +1098,7 @@ python3_mapping = {
 def _overlay_mapping(mapping, new_entries):
   ret = mapping.copy()
   ret.update(new_entries)
-  return dict((k, v) for k, v in six.iteritems(ret) if v is not None)
-
-python_3_5_mapping = _overlay_mapping(python3_mapping, {
-    16: BINARY_MATRIX_MULTIPLY,
-    17: INPLACE_MATRIX_MULTIPLY,
-    54: None,
-    69: GET_YIELD_FROM_ITER,  # STORE_LOCALS in Python 3.3
-    81: WITH_CLEANUP_START,  # WITH_CLEANUP in Python 3.4
-    82: WITH_CLEANUP_FINISH,
-    149: BUILD_LIST_UNPACK,
-    150: BUILD_MAP_UNPACK,
-    151: BUILD_MAP_UNPACK_WITH_CALL,
-    152: BUILD_TUPLE_UNPACK,
-    153: BUILD_SET_UNPACK,
-})
+  return dict((k, v) for k, v in ret.items() if v is not None)
 
 python_3_6_mapping = _overlay_mapping(python_3_5_mapping, {
     85: SETUP_ANNOTATIONS,
@@ -1104,16 +1117,29 @@ python_3_7_mapping = _overlay_mapping(python_3_6_mapping, {
     161: CALL_METHOD,
 })
 
+python_3_8_mapping = _overlay_mapping(python_3_7_mapping, {
+    6: ROT_FOUR,  # ROT_FOUR returns under a different, cooler id!
+    53: BEGIN_FINALLY,
+    54: END_ASYNC_FOR,
+    80: None,   # BREAK_LOOP was removed in 3.8
+    119: None,  # CONTINUE_LOOP was removed in 3.8
+    120: None,  # SETUP_LOOP was removed in 3.8
+    121: None,  # SETUP_EXCEPT was removed in 3.8
+    162: CALL_FINALLY,
+    163: POP_FINALLY,
+})
 
-class _LineNumberTableParser(object):
+
+class _LineNumberTableParser:
   """State machine for decoding a Python line number array."""
 
-  def __init__(self, lnotab, firstlineno):
+  def __init__(self, python_version, lnotab, firstlineno):
     assert not len(lnotab) & 1  # lnotab always has an even number of elements
     self.lnotab = lnotab
     self.lineno = firstlineno
     self.next_addr = six.indexbytes(self.lnotab, 0) if self.lnotab else 0
     self.pos = 0
+    self.python_version = python_version
 
   def get(self, i):
     """Get the line number for the instruction at the given position.
@@ -1128,7 +1154,14 @@ class _LineNumberTableParser(object):
       The line number corresponding to the position at i.
     """
     while i >= self.next_addr and self.pos < len(self.lnotab):
-      self.lineno += six.indexbytes(self.lnotab, self.pos + 1)
+      line_diff = six.indexbytes(self.lnotab, self.pos + 1)
+      # The Python docs have more details on this weird bit twiddling.
+      # https://github.com/python/cpython/blob/master/Objects/lnotab_notes.txt
+      # https://github.com/python/cpython/commit/f3914eb16d28ad9eb20fe5133d9aa83658bcc27f
+      if self.python_version >= (3, 6) and line_diff >= 0x80:
+        line_diff -= 0x100
+      self.lineno += line_diff
+
       self.pos += 2
       if self.pos < len(self.lnotab):
         self.next_addr += six.indexbytes(self.lnotab, self.pos)
@@ -1229,12 +1262,15 @@ def _wordcode_reader(data, mapping):
       start = pos + 2
 
 
-def _dis(data, mapping, reader,
+def _dis(python_version, data, mapping, reader,
          co_varnames=None, co_names=None, co_consts=None, co_cellvars=None,
          co_freevars=None, co_lnotab=None, co_firstlineno=None):
   """Disassemble a string into a list of Opcode instances."""
   code = []
-  lp = _LineNumberTableParser(co_lnotab, co_firstlineno) if co_lnotab else None
+  if co_lnotab:
+    lp = _LineNumberTableParser(python_version, co_lnotab, co_firstlineno)
+  else:
+    lp = None
   offset_to_index = {}
   if co_cellvars is not None and co_freevars is not None:
     cellvars_freevars = co_cellvars + co_freevars
@@ -1275,13 +1311,13 @@ def dis(data, python_version, *args, **kwargs):
   assert major in (2, 3)
   mapping = {
       (2, 7): python2_mapping,
-      (3, 4): python3_mapping,
       (3, 5): python_3_5_mapping,
       (3, 6): python_3_6_mapping,
       (3, 7): python_3_7_mapping,
+      (3, 8): python_3_8_mapping
   }[(major, minor)]
   reader = _wordcode_reader if (major, minor) > (3, 5) else _bytecode_reader
-  return _dis(data, mapping, reader, *args, **kwargs)
+  return _dis(python_version, data, mapping, reader, *args, **kwargs)
 
 
 def dis_code(code):
